@@ -1,15 +1,19 @@
+from . import checked_threading
 from . import configuration
 from . import editor
 from . import qemu
 from . import image
 from . import utils
+from . import scan
 from . import ssh
 from . import sshfs
 from . import static
 
+import base64
 import contextlib
 import enum
 import glob
+import json
 import logging
 import os
 import pwd
@@ -40,6 +44,9 @@ else:
     Environ = os._Environ
 
 
+MAX_CONCURRENT_SSHFS = 7
+
+
 @enum.unique
 class TransientVmState(enum.Enum):
     WAITING = (1,)
@@ -52,6 +59,7 @@ class TransientVm:
     config: configuration.Config
     vm_images: Sequence[image.BaseImageInfo]
     ssh_config: Optional[ssh.SshConfig]
+    set_ssh_port: Optional[int]
     qemu_runner: Optional[qemu.QemuRunner]
     qemu_should_die: bool
 
@@ -64,6 +72,7 @@ class TransientVm:
         self.qemu_should_die = False
         self.name = self.config.name or self.__generate_tmp_name()
         self.state = TransientVmState.WAITING
+        self.set_ssh_port = None
 
     def __generate_tmp_name(self) -> str:
         return str(uuid.uuid4())
@@ -205,8 +214,14 @@ class TransientVm:
         if self.__is_stateless():
             new_args.append("-snapshot")
 
-        for image in self.vm_images:
-            new_args.extend(["-drive", f"file={image.path}"])
+        if self.config.no_virtio_scsi:
+            for image in self.vm_images:
+                new_args.extend(["-drive", f"file={image.path}"])
+        else:
+            new_args.extend(["-device", "virtio-scsi-pci,id=scsi"])
+            for idx, image in enumerate(self.vm_images):
+                new_args.extend(["-drive", f"file={image.path},if=none,id=hd{idx}"])
+                new_args.extend(["-device", f"scsi-hd,drive=hd{idx}"])
 
         if self.__needs_ssh():
             if self.__needs_ssh_console():
@@ -216,21 +231,24 @@ class TransientVm:
                 ssh_port = utils.allocate_random_port()
             else:
                 ssh_port = self.config.ssh_port
+            self.set_ssh_port = ssh_port
 
             self.ssh_config = ssh.SshConfig(
                 host="127.0.0.1",
-                port=ssh_port,
+                port=self.set_ssh_port,
                 user=self.config.ssh_user,
                 ssh_bin_name=self.config.ssh_bin_name,
             )
 
+            ssh_net_driver = self.config.ssh_net_driver
+
             # the random localhost port or the user provided port to guest port 22
             new_args.extend(
                 [
                     "-netdev",
                     f"user,id=transient-sshdev,hostfwd=tcp::{ssh_port}-:22",
                     "-device",
-                    "e1000,netdev=transient-sshdev",
+                    f"{ssh_net_driver},netdev=transient-sshdev",
                 ]
             )
 
@@ -312,6 +330,18 @@ class TransientVm:
             raise utils.TransientProcessError(returncode=returncode)
         return None
 
+    def __build_qemu_environment(self) -> Dict[str, str]:
+        env = {scan.SCAN_ENVIRON_SENTINEL: "1"}
+        data = {"name": self.name}
+
+        if self.set_ssh_port is not None:
+            data["ssh_port"] = self.set_ssh_port
+
+        data_json_bytes = json.dumps(data).encode("utf-8")
+        data_encoded = base64.b64encode(data_json_bytes).decode("utf-8")
+        env[scan.SCAN_ENVIRON_DATA] = data_encoded
+        return env
+
     def run(self) -> None:
         self.state = TransientVmState.RUNNING
 
@@ -347,9 +377,11 @@ class TransientVm:
         # `-nographic` mode, which is very surprising.
         self.qemu_runner = qemu.QemuRunner(
             full_qemu_args,
+            bin_name=self.config.qemu_bin_name,
             quiet=qemu_quiet,
             interactive=qemu_interactive,
             qmp_connectable=self.__needs_ssh_console(),
+            env=self.__build_qemu_environment(),
         )
 
         qemu_proc = self.qemu_runner.start()
@@ -362,6 +394,7 @@ class TransientVm:
             logging.error("QEMU Process has died. Exiting")
             return self.__post_run(qemu_returncode)
 
+        sshfs_threads = []
         for shared_spec in self.config.shared_folder:
             assert self.ssh_config is not None
             local, remote = shared_spec.split(":")
@@ -369,13 +402,25 @@ class TransientVm:
             # The user almost certainly doesn't intend to pass a relative path,
             # so make it absolute
             absolute_local_path = os.path.abspath(local)
-            sshfs.do_sshfs_mount(
-                connect_timeout=self.config.ssh_timeout,
-                ssh_config=self.ssh_config,
-                local_dir=absolute_local_path,
-                remote_dir=remote,
-                local_user=self.__current_user(),
+            sshfs_kwargs = {
+                "connect_timeout": self.config.ssh_timeout,
+                "ssh_config": self.ssh_config,
+                "local_dir": absolute_local_path,
+                "remote_dir": remote,
+                "local_user": self.__current_user(),
+            }
+
+            # Slamming the server with 20 connections at once is a good way to break things:
+            if len(sshfs_threads) > MAX_CONCURRENT_SSHFS:
+                sshfs_threads.pop(0).join()
+
+            sshfs_threads.append(
+                checked_threading.Thread(target=sshfs.do_sshfs_mount, kwargs=sshfs_kwargs)
             )
+            sshfs_threads[-1].start()
+
+        for sshfs_thread in sshfs_threads:
+            sshfs_thread.join()
 
         if self.__needs_ssh_console():
             # Now wait until the QMP connection is established (this should be very fast).
